; ________________________________________________
; |              Macro generiche PIC             |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Tipo      : INC                            |
; |   Versione  : 1                              |
; |   Release   : 0                              |
; |   Autore    : Pinton Fabio, mod. Lain Walter |
; |   Data      : 24/12/06                       |
; |______________________________________________|

        NOLIST
;****************************************************************************
;                                   MACRO
;****************************************************************************

;============================================================================
;   Disabilita Interrupts
;============================================================================
CLI     MACRO
        BCF INTCON,GIE
        ENDM

;============================================================================
;   Abilita Interrupts
;============================================================================
SEI     MACRO
        BSF INTCON,GIE
        ENDM

;============================================================================
;   Salvataggio registri
;============================================================================
PUSH    MACRO
        MOVWF WINT                      ;Salva W
        SWAPF WINT,F                     ;Swappa per POP
        SWAPF STATUS,W                  ;Salva STATUS
        MOVWF SINT                       ;
        MOVF FSR,W                      ;Salva FSR
        MOVWF FINT                       ;
        MOVF PCLATH,W                   ;Salva PCLATH
        MOVWF PINT                       ;
        CLRF STATUS                     ;Resetta STATUS (Bank 0)
        CLRF PCLATH                     ;Resetta PCLATH (Page 0)
        ENDM

;============================================================================
;   Ripristino registri
;============================================================================
POP     MACRO
        MOVF PINT,W                     ;PCLATH
        MOVWF PCLATH                     ;
        MOVF FINT,W                     ;FSR
        MOVWF FSR                        ;
        SWAPF SINT,W                    ;STATUS
        MOVWF STATUS                     ;
        SWAPF WINT,W                    ;W
        ENDM

;============================================================================
;   Selezione RP (Indirizzamento diretto)
;============================================================================

BANKSET MACRO Num

        IF Num == 0                     ;Seleziona il Bank 0
        BCF STATUS,RP0                   ;
        BCF STATUS,RP1                   ;
        ENDIF                            ;

        IF Num == 1                     ;Seleziona il Bank 1
        BSF STATUS,RP0                   ;
        BCF STATUS,RP1                   ;
        ENDIF                            ;

        IF Num == 2                     ;Seleziona il Bank 2
        BCF STATUS,RP0                   ;
        BSF STATUS,RP1                   ;
        ENDIF                            ;

        IF Num == 3                     ;Seleziona il Bank 3
        BSF STATUS,RP0                   ;
        BSF STATUS,RP1                   ;
        ENDIF                            ;

        ENDM

;============================================================================
;   Selezione IRP (Indirizzamento Indiretto)
;============================================================================

BANKISET MACRO Num

        IF Num==0 || Num==1             ;Seleziona Bank 0 / Bank 1
        BCF STATUS,IRP                   ;
        ELSE                            ;
        BSF STATUS,IRP                  ;Seleziona Bank 2 / Bank 3

        ENDM

;============================================================================
;   AND da file a file
;============================================================================
ANDFF MACRO File1,File2

        MOVF File2,W                    ;Carica File2
        ANDWF File1,F                   ;AND e risultato in File1

        ENDM

;============================================================================
;   Muove da file a file
;============================================================================

MOVFF   MACRO Sorg,Dest

        MOVF Sorg,W                     ;Carica Sorgente
        MOVWF Dest                      ;Scrive in Destinazione

        ENDM

;============================================================================
;   Muove costante a file
;============================================================================

MOVLF   MACRO Lab,Dest

        MOVLW Lab                       ;Carica Sorgente
        MOVWF Dest                      ;Scrive in destinazione

        ENDM

;============================================================================
;   Muove costante a word
;============================================================================
MOVL2W  MACRO Lab,Dest

        MOVLW HIGH Lab                  ;Carica MSB
        MOVWF Dest                       ;
        MOVLW LOW Lab                   ;Carica LSB
        MOVWF Dest+1                     ;

        ENDM

;****************************************************************************
;                            ISTRUZIONI A WORD
;****************************************************************************
;------------------------------------------------------------------------------
;   Test
;------------------------------------------------------------------------------
TSTW    MACRO What

        TSTF What                       ;Testa MSB
        SKPNZ                            ;
        TSTF What+1                     ;Testa LSB

        ENDM

;------------------------------------------------------------------------------
;   Decrementa
;------------------------------------------------------------------------------
DECW    MACRO What

        TSTF What+1                     ;LSB not null ?
        SKPNZ                            ;
        DECF What,F                      ;SI: Descremnta MSB
        DECF What+1,F                   ;Decrementa LSB

        ENDM

;------------------------------------------------------------------------------
;   Incrementa
;------------------------------------------------------------------------------
INCW    MACRO What

        INCF  What+1,F                  ;Incrementa LSB
        SKPNZ                           ;Riporto ?
        INCF  What,F                     ;SI: Incrementa MSB

        ENDM

;------------------------------------------------------------------------------
;    Somma Words -> Wo_A = Wo_A + Wo_B
;------------------------------------------------------------------------------
ADDWB   MACRO Wo_A,By_B

        MOVFW By_B                      ;Somma LSB
        ADDWF Wo_A+1,F                   ;
        SKPNC                           ;Riporto ?
        INCF Wo_A,F                      ;SI: Incrementa MSB

        ENDM

;------------------------------------------------------------------------------
;    Somma Words -> Wo_A = Wo_A + Wo_B
;------------------------------------------------------------------------------
ADDW    MACRO Wo_A,Wo_B

        MOVFW Wo_B+1                    ;Somma LSB
        ADDWF Wo_A+1,F                   ;
        SKPNC                           ;Riporto ?
        INCF Wo_A,F                      ;SI: Incrementa MSB
        MOVFW Wo_B                      ;Somma MSB
        ADDWF Wo_A,F                     ;

        ENDM

;------------------------------------------------------------------------------
;    Somma Word con Literal -> Wo = Wo + Lt
;------------------------------------------------------------------------------
ADDWL   MACRO Wo,Lt

        MOVLW LOW Lt                     ;Somma LSB
        ADDWF Wo+1,F                     ;
        SKPNC                           ;Riporto ?
        INCF Wo,F                        ;SI: Incrementa MSB
        MOVLW HIGH Lt                   ;Somma MSB
        ADDWF Wo,F                       ;

        ENDM

;------------------------------------------------------------------------------
;    Sottrae Words -> Wo_A = Wo_A - Wo_B
;------------------------------------------------------------------------------
SUBW    MACRO Wo_A,Wo_B

        MOVFW Wo_B+1                    ;Sottrae LSB
        SUBWF Wo_A+1,F                   ;
        SKPC                            ;Riporto ?
        DECF Wo_A,F                      ;SI: Decrementa MSB
        MOVFW Wo_B                      ;Sottrae MSB
        SUBWF Wo_A,F                     ;

        ENDM

;------------------------------------------------------------------------------
;    Sottrae Literal alla Word -> Wo = Wo - Lt
;------------------------------------------------------------------------------
SUBWL   MACRO Wo,Lt

        MOVLW LOW Lt                    ;Sottrae LSB
        SUBWF Wo+1,F                     ;
        SKPC                            ;Riporto ?
        DECF Wo,F                        ;SI: Decrementa MSB
        MOVLW HIGH Lt                   ;Sottrae MSB
        SUBWF Wo,F                       ;

        ENDM

;****************************************************************************
;                               CONFRONTI
;****************************************************************************
;------------------------------------------------------------------------------
;   Confronta due byte (By_B - By_A)
;------------------------------------------------------------------------------
CMPB    MACRO By_A,By_B

        MOVFW By_A                      ;Carica Byte
        SUBWF By_B,W                    ;Sottrae in W

        ENDM

;------------------------------------------------------------------------------
;   Confronta byte / literal (Lt - By)
;------------------------------------------------------------------------------
CMPBL   MACRO By,Lt

        MOVFW By                        ;Carica Byte
        SUBLW Lt                        ;Sottrae

        ENDM

;------------------------------------------------------------------------------
;   Confronta due word (Wo_B - Wo_A)
;------------------------------------------------------------------------------
CMPW    MACRO Wo_A,Wo_B

        MOVFW Wo_A                      ;Confronta MSB
        SUBWF Wo_B,W                     ;

        BNC $+5                         ;Wo_B < Wo_A
        BNZ $+3                         ;Wo_B > Wo_A

        MOVFW Wo_A+1                    ;Confronta LSB
        SUBWF Wo_B+1,W                   ;

        ENDM

;****************************************************************************
;                             SALTI SU CONDIZIONE
;****************************************************************************
;L'assemblatore di Microchip prevede delle macro di salto:
;   B       :       Branch
;   BC      :       Branch on Carry
;   BDC     :       Branch on Digit Carry
;   BNC     :       Branch on No Carry
;   BNDC    :       Branch on No Digit Carry
;   BNZ     :       Branch on No Zero
;   BZ      :       Branch on Zero
;
;Vengono implementare delle "istruzioni" aggiuntive di branch su operazioni
; di confronto tra due numeri
;   BEQ     :       Branch if Equal
;   BNE     :       Branch if Not Equal
;   BG      :       Branch if Great
;   BGE     :       Branch if Great or Equal
;   BL      :       Branch if Less
;   BLE     :       Branch if Less or Equal

;------------------------------------------------------------------------------
;   Branch if Equal
;------------------------------------------------------------------------------
BEQ     MACRO label

        BZ label

        ENDM

;------------------------------------------------------------------------------
;   Branch if Not Equal
;------------------------------------------------------------------------------
BNE     MACRO label

        BNZ label

        ENDM

;------------------------------------------------------------------------------
;   Branch if Great
;------------------------------------------------------------------------------
BG      MACRO label

        BZ $+3
        BC label

        ENDM

;------------------------------------------------------------------------------
;   Branch if Great or Equal
;------------------------------------------------------------------------------
BGE     MACRO label

        BC label

        ENDM

;------------------------------------------------------------------------------
;   Branch if Less
;------------------------------------------------------------------------------
BL      MACRO label

        BZ $+3
        BNC label

        ENDM

;------------------------------------------------------------------------------
;   Branch if Less or Equal
;------------------------------------------------------------------------------
BLE     MACRO label

        BZ label
        BNC label

        ENDM

    LIST

